/*
 * bmp180.c
 *    Reads the Bosch bmp180 temperature and pressure sensor
 *    Makes use of the bmp180api written by Bosch and compiled into a 
 *    shared library.
 *    Copyright U. Raich Jan. 2017
 *    This program is part of a course on embedded systems held
 *    at the University of Cape Coast, Ghana in 2017
 *    It is released under the GNU Public License 
 *    For details please see https://www.gnu.org/licenses/gpl.html
 */

#include <stdio.h>
#include <bmp180.h>

#define Pa2mbar ((double)1013.15 /(double)101325)
extern s8 I2C_routine(void);
extern struct bmp180_t bmp180;

void printBmp180Struct(void) {
  printf("\nbmp180 structure:\n");
  printf("mode:        	      %d\n",bmp180.mode);
  printf("chip_id:     	   0x%02x\n",bmp180.chip_id);
  printf("ml_version:  	      %d\n",bmp180.ml_version);
  printf("al_version:  	      %d\n",bmp180.al_version);
  printf("dev_addr:    	   0x%02x\n",bmp180.dev_addr);
  printf("sensortype:  	      %d\n",bmp180.sensortype);
  printf("param_b5:    	      %d\n",bmp180.param_b5);
  printf("number_os_samples:    %d\n",bmp180.number_of_samples);
  printf("oversamp_setting:     %d\n",bmp180.oversamp_setting);
  printf("sw_oversamp:          %d\n\n",bmp180.sw_oversamp);
}

void printBmp180Calibration(void) {
  printf("\nbmp180 calibration values:\n");
  printf("ac1:   %d\n",bmp180.calib_param.ac1);
  printf("ac2:   %d\n",bmp180.calib_param.ac2);
  printf("ac3:   %d\n",bmp180.calib_param.ac3);
  printf("ac4:   %d\n",bmp180.calib_param.ac4);
  printf("ac5:   %d\n",bmp180.calib_param.ac5);
  printf("ac6:   %d\n",bmp180.calib_param.ac6);
  printf("b1:    %d\n",bmp180.calib_param.b1);
  printf("b2:    %d\n",bmp180.calib_param.b2);
  printf("mb:    %d\n",bmp180.calib_param.mb);
  printf("mc:    %d\n",bmp180.calib_param.mc);
  printf("md:    %d\n\n",bmp180.calib_param.md);
}
int main(int argc, char ** argv) {

  BMP180_RETURN_FUNCTION_TYPE bmp180RetCode;
  //  u8 bmp180StructInit=0xff;
  u16 rawTemp;
  u32 rawPress;
  double temperature;
  u32 pressure;
  //  bmp180StructInit=I2C_routine();
  I2C_routine();
  printf("bmp180: bmp180 structure successfully initialized\n");
  if((bmp180RetCode = bmp180_init(&bmp180)) < 0)
    {
      printf("bmp180: Could not initialize bmp180 chip\n");
      return -1;
    }
  /*
    set to ultra high resolution for pressure measurement and
    average over 3 measurements
  */
  bmp180.oversamp_setting=BMP180_OVERSAMP_SETTING_U8X;
  bmp180.sw_oversamp=BMP180_SW_OVERSAMP_U8X;
  //  printBmp180Struct();
  printBmp180Calibration();
  printf("bmp180: Chip successfully initialized\n");
  //  bmp180_debug(true);
  rawTemp = bmp180_get_uncomp_temperature();
  printf("bmp180: raw temperature: %d\n",rawTemp);
  temperature = (double)bmp180_get_temperature(rawTemp)/10.0;
  printf("bmp180: temperature: %5.1f °C\n",temperature);
  printBmp180Struct();

  /*
    now for barometric pressure
  */
  rawPress = bmp180_get_uncomp_pressure(); 
  pressure = bmp180_get_pressure(rawPress);
  printf("bmp180: raw pressure: %d\n",rawPress);
  /*
   101325 Pa <=> 1.01325 bar <=>760 Torr
  */
  printf("bmp180: pressure: %10.2f hPa\n",(double)pressure/100.0);
  return 0;
}
