/* hd44780.h Header file for the Hitachi LCD display driver
 *    Copyright U. Raich Jan. 2017
 *    This program is part of a course on embedded systems held
 *    at the University of Cape Coast, Ghana in 2017
 *    It is released under the GNU Public License 
 *    For details please see https://www.gnu.org/licenses/gpl.html
 */
#ifndef __HD44780_H__
#define __HD44780_H__

#include <stdio.h>
#include <pigpiod_if2.h>
#include <errno.h>
#include <unistd.h>
#include <stdbool.h>

#define I2C_BUS                   1
#define I2C_FLAGS                 0
#define HD44780_ADDRESS        0x3f

#define HD44780_CLEAR          0x01
#define HD44780_HOME           0x02
#define HD44780_ENTRY_MODE     0x04
#define HD44780_CURSOR_DIR     0x02
#define HD44780_CURSOR_SHIFT   0x10
#define HD44780_DISPLAY_CURSOR 0x08
#define HD44780_DISPLAY_ON     0x04
#define HD44780_DISPLAY_OFF    0x00
#define HD44780_CURSOR_ON      0x02
#define HD44780_CURSOR_OFF     0x00
#define HD44780_CURSOR_BLINK   0x01
#define HD44780_CURSOR_NOBLINK 0x00

#define HD44780_ENTIRE_DISPLAY 0x04
#define HD44780_CURSOR_ONOFF   0x02
#define HD44780_BLINK          0x01
#define HD44780_CURSOR         0x10
#define HD44780_1_LINE         0x00
#define DH44780_CURSOR_MOVE    0x00
#define DH44780_DISPLAY_SHIFT  0x08
#define DH44780_SHIFT_LEFT     0x00
#define DH44780_SHIFT_RIGHT    0x04

#define HD44780_FUNCTION       0x20

#define HD44789_8_BIT_MODE     0x10
#define HD44780_4_BIT_MODE     0x00
#define HD44780_TWO_LINES      0x08
#define HD44780_5x10_FONT      0x04
#define HD44780_5x8_FONT       0x00
#define HD44780_1_LINE         0x00
#define HD44780_2_LINE         0x08
#define HD44780_CURSOR_DEC     0x00
#define HD44780_CURSOR_INC     0x02
#define HD44780_DISPLAY_SHIFT  0x01

#define HD44780_CGRAM_AHDR     0x10
#define HD44780_HDRAM_AHDR     0x20
#define HD44780_BUSY_FLAG      0x40

#define HD44780_BL             0x08
#define HD44780_EN             0x04
#define HD44780_READ           0x02
#define HD44780_WRITE          0x00
#define HD44780_CMD_REG        0x00
#define HD44780_DATA_REG       0x01

#define HD44780_SET_DDRAM      0x80
#define HD44780_2ND_LINE       0x40

#define HD44780_INCREMENT      true
#define HD44780_DECREMENT      false
#define HD44780_ON             true
#define HD44780_OFF            false

#define HD44780_SUCCESS        0
#define HD44780_MAX_STRLEN     16

/*
pinout for 20x2 LCD via PCF8574AN:
----------
0x80    P7 -  - D7
0x40    P6 -  - D6
0x20    P5 -  - D5
0x10    P4 -  - D4
-----------
0x08    P3 -  - BL   Backlight (LCD has 3 RGB LEDs - common anode)
0x04    P2 -  - EN   Starts Data read/write
0x02    P1 -  - RW   low: write, high: read
0x01    P0 -  - RS   Register Select: 0: Instruction Register (IR) (AC when read), 
                                      1: data register (DR)
*/

/*
  function prototypes
*/
void  hd44780SetDebug(bool);
int   hd44780BackLight(bool);
int   hd44780PutC(char);
int   hd44780GetC(char *);
int   hd44780ReadBusy(bool *, unsigned char *);  
int   hd44780PutS(char *);
int   hd44780Home(void);
int   hd44780Clear(void);
int   hd44780Cursor(bool,bool);
int   hd44780EntryMode(bool,bool);
int   hd44780SetDDRamAddr(unsigned char);
int   hd44780SecondLine(void);
int   hd44780Open(void);
void  hd44780Close(void);

#endif



